@@ -1,13 +1,11 @@
 """Module provider for Godaddy"""
-from __future__ import absolute_import
-
 import hashlib
 import json
 import logging
 
 import requests
 from requests.adapters import HTTPAdapter
-from urllib3.util.retry import Retry
+from urllib3.util.retry import Retry  # type: ignore
 
 from lexicon.providers.base import Provider as BaseProvider
 
